package icu.fur93.esp32_car

import android.Manifest
import androidx.compose.ui.graphics.Color
import android.app.Activity
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.WindowManager
import androidx.activity.ComponentActivity
import androidx.activity.SystemBarStyle
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.annotation.RequiresApi
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.Send
import androidx.compose.material.icons.automirrored.outlined.Send
import androidx.compose.material.icons.filled.Home
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material.icons.outlined.Home
import androidx.compose.material.icons.outlined.Settings
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.toArgb
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalView
import androidx.compose.ui.tooling.preview.Preview
import androidx.core.app.ActivityCompat
import androidx.core.view.WindowCompat
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navigation
import icu.fur93.esp32_car.data.PreferencesDataStore
import icu.fur93.esp32_car.page.ControlGamepadModePage
import icu.fur93.esp32_car.page.ControlPage
import icu.fur93.esp32_car.page.ControlPathfinderModePage
import icu.fur93.esp32_car.page.ControlSingleJoystickModePage
import icu.fur93.esp32_car.page.HomePage
import icu.fur93.esp32_car.page.SettingsPage
import icu.fur93.esp32_car.provider.LocalNavigation
import icu.fur93.esp32_car.repository.BluetoothRepositoryImpl
import icu.fur93.esp32_car.ui.component.BottomNavigationBar
import icu.fur93.esp32_car.ui.component.BottomNavigationItem
import icu.fur93.esp32_car.ui.theme.Esp32carTheme
import icu.fur93.esp32_car.viewmodel.CarControlUseCase
import icu.fur93.esp32_car.viewmodel.CarViewModel
import kotlinx.coroutines.launch

class MainActivity : ComponentActivity() {
    private val preferencesDataStore by lazy {
        PreferencesDataStore(this)
    }

    private val bluetoothRepository by lazy {
        BluetoothRepositoryImpl(this, lifecycleScope, preferencesDataStore)
    }

    private val carControlUseCase by lazy {
        CarControlUseCase(bluetoothRepository)
    }

    private val viewModel by viewModels<CarViewModel> {
        object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return CarViewModel(carControlUseCase, bluetoothRepository) as T
            }
        }
    }

    @RequiresApi(Build.VERSION_CODES.S)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge(
            statusBarStyle = SystemBarStyle.auto(
                lightScrim = Color.Transparent.toArgb(),  // 浅色主题下状态栏的微弱遮罩
                darkScrim = Color.Transparent.toArgb(),   // 深色主题下状态栏的微弱遮罩
            ),
            navigationBarStyle = SystemBarStyle.auto(
                lightScrim = Color.Transparent.toArgb(),  // 浅色主题下导航栏的微弱遮罩
                darkScrim = Color.Transparent.toArgb(),   // 深色主题下导航栏的微弱遮罩
            )
        )

        // 设置全屏显示
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            window.attributes.layoutInDisplayCutoutMode =
                WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

        // 检查并请求所有必要的权限
        val requiredPermissions = arrayOf(
            Manifest.permission.BLUETOOTH_SCAN,
            Manifest.permission.BLUETOOTH_CONNECT,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        )

        val missingPermissions = requiredPermissions.filter {
            ActivityCompat.checkSelfPermission(this, it) != PackageManager.PERMISSION_GRANTED
        }

        if (missingPermissions.isNotEmpty()) {
            ActivityCompat.requestPermissions(
                this,
                missingPermissions.toTypedArray(),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
        }

        // 检查权限后尝试自动连接
        lifecycleScope.launch {
            preferencesDataStore.getLastConnectedDevice().collect { address ->
                if (address != null) {
                    viewModel.connectToDeviceByAddress(address)
                }
            }

        }

        setContent {
            Esp32carTheme {
                App(viewModel)
            }
        }
    }

    // 添加权限请求结果处理
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == BLUETOOTH_PERMISSION_REQUEST_CODE) {
            if (grantResults.all { it == PackageManager.PERMISSION_GRANTED }) {
                Log.d("MainActivity", "所有权限已获得")
            } else {
                Log.e("MainActivity", "部分权限未获得")
            }
        }
    }

    companion object {
        private const val BLUETOOTH_PERMISSION_REQUEST_CODE = 1
    }
}

@Composable
fun App(viewModel: CarViewModel) {

    val navController = rememberNavController()

    val isDarkTheme = isSystemInDarkTheme()
    val view = LocalView.current
    val window = (LocalContext.current as? MainActivity)?.window

    DisposableEffect(isDarkTheme) {
        window?.apply {
            // 2. 设置导航栏颜色为透明
            navigationBarColor = Color.Transparent.toArgb()
            // 3. 根据主题设置导航栏图标颜色
            WindowCompat.getInsetsController(this, view).apply {
                isAppearanceLightNavigationBars = !isDarkTheme
            }
        }
        onDispose {}
    }

    CompositionLocalProvider(
        LocalNavigation provides navController,
    ) {
        Scaffold(
            bottomBar = {
                BottomNavigationBar(
                    navController = navController,
                    navItems = listOf(
                        BottomNavigationItem(
                            icon = Icons.Outlined.Home,
                            selectedIcon = Icons.Default.Home,
                            route = Route.Home
                        ),
                        BottomNavigationItem(
                            icon = Icons.AutoMirrored.Outlined.Send,
                            selectedIcon = Icons.AutoMirrored.Filled.Send,
                            route = Route.Control
                        ),
                        BottomNavigationItem(
                            icon = Icons.Outlined.Settings,
                            selectedIcon = Icons.Default.Settings,
                            route = Route.Settings
                        )
                    )
                )
            },
            modifier = Modifier
                .fillMaxSize(),
            containerColor = MaterialTheme.colorScheme.surface,
        ) { innerPadding ->
            Box(Modifier.padding(innerPadding)) {
                NavHost(
                    navController = navController,
                    startDestination = Route.Home.route
                ) {
                    // 主导航路由
                    composable(Route.Home.route) { HomePage(viewModel) }
                    composable(Route.Settings.route) { SettingsPage(viewModel) }

                    // Control 相关的导航图
                    navigation(
                        startDestination = Route.Control.route,
                        route = "control_graph"
                    ) {
                        composable(Route.Control.route) { ControlPage(viewModel) }

                        // Control 的子页面
                        composable(Route.ControlPathfinderMode.route) {
                            ControlPathfinderModePage(viewModel)
                        }
                        composable(Route.ControlSingleJoystickMode.route) {
                            ControlSingleJoystickModePage(viewModel)
                        }
                        composable(Route.ControlGamepadMode.route) {
                            ControlGamepadModePage(viewModel)
                        }
                    }
                }
            }
        }
    }
}