package icu.fur93.esp32_car.page

import android.view.MotionEvent
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.FilledIconButton
import androidx.compose.material3.Icon
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.ExperimentalComposeUiApi
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.pointer.pointerInteropFilter
import androidx.compose.ui.res.vectorResource
import androidx.compose.ui.unit.dp
import icu.fur93.esp32_car.R
import icu.fur93.esp32_car.Route
import icu.fur93.esp32_car.entity.control.JoystickState
import icu.fur93.esp32_car.ui.carditem.StatusCardInfo
import icu.fur93.esp32_car.ui.carditem.StatusCardJoystickStatus
import icu.fur93.esp32_car.ui.carditem.StatusCardMotorStatus
import icu.fur93.esp32_car.ui.component.Joystick
import icu.fur93.esp32_car.ui.component.PageTitle
import icu.fur93.esp32_car.ui.component.StatusCard
import icu.fur93.esp32_car.ui.theme.LayoutContentModifier
import icu.fur93.esp32_car.viewmodel.CarViewModel
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

@Composable
fun ControlSingleJoystickModePage(viewModel: CarViewModel) {
    var joystickState by remember { mutableStateOf(JoystickState()) }

    Column(LayoutContentModifier) {
        PageTitle(Route.ControlSingleJoystickMode.title)
        ControlSingleJoystickModeStatusCard(viewModel, joystickState)
        ControlSingleJoystickModeControlPanel(viewModel) {
            joystickState = it
        }
    }

}

@Composable
fun ControlSingleJoystickModeStatusCard(viewModel: CarViewModel, joystickState: JoystickState) {
    val deviceInfo by viewModel.connectionInfoState.collectAsState()
    val carState by viewModel.carState.collectAsState()

    StatusCard(
        cardItems = listOf(
            StatusCardInfo(deviceInfo),
            StatusCardMotorStatus(carState),
            StatusCardJoystickStatus(joystickState)
        )
    )
}

@OptIn(ExperimentalComposeUiApi::class)
@Composable
fun ControlSingleJoystickModeControlPanel(
    viewModel: CarViewModel,
    onJoystickStateChange: (JoystickState) -> Unit
) {

    Spacer(Modifier.height(24.dp))

    Row(
        modifier = Modifier.fillMaxWidth(),
        horizontalArrangement = Arrangement.SpaceBetween,
        verticalAlignment = Alignment.CenterVertically
    ) {
        FilledIconButton(
            onClick = {},
            modifier = Modifier.pointerInteropFilter { event ->
                when (event.action) {
                    MotionEvent.ACTION_DOWN -> {
                        viewModel.sendXYR(0, 0, -50)
                        true
                    }
                    MotionEvent.ACTION_UP -> {
                        viewModel.stop()
                        true
                    }
                    else -> false
                }
            }
        ) {
            Icon(ImageVector.vectorResource(R.drawable.rotate_left_24), "逆时针旋转")
        }

        ControlJoystick(
            viewModel = viewModel,
            onJoystickStateChange = onJoystickStateChange
        )

        FilledIconButton(
            onClick = {},
            modifier = Modifier.pointerInteropFilter { event ->
                when (event.action) {
                    MotionEvent.ACTION_DOWN -> {
                        viewModel.sendXYR(0, 0, 50)
                        true
                    }
                    MotionEvent.ACTION_UP -> {
                        viewModel.stop()
                        true
                    }
                    else -> false
                }
            }
        ) {
            Icon(ImageVector.vectorResource(R.drawable.rotate_right_24), "顺时针旋转")
        }
    }
}

@Composable
fun ControlJoystick(
    viewModel: CarViewModel,
    onJoystickStateChange: (JoystickState) -> Unit
) {
    // 记住上一次发送的状态和时间
    val lastSentState = remember { mutableStateOf(JoystickState()) }
    val lastSentTime = remember { mutableStateOf(0L) }

    // 用于停止命令重发的计数器
    val stopCommandCount = remember { mutableStateOf(0) }
    val coroutineScope = rememberCoroutineScope()



    Joystick(
        size = 175.dp,
        knobScale = 0.25f
    ) { state ->
        // 更新状态
        onJoystickStateChange(state)

        val currentTime = System.currentTimeMillis()

        // 检查是否是停止状态（摇杆回正）
        val isStopCommand = state.x == 0f && state.y == 0f

        // 如果是停止命令，启动重发机制
        // 否则检查时间间隔和状态变化
        if (isStopCommand) {
            if (stopCommandCount.value == 0) {
                // 第一次发送停止命令
                viewModel.stop()
                stopCommandCount.value = 1

                // 启动延时重发
                coroutineScope.launch {
                    delay(50) // 延时50ms
                    viewModel.stop() // 再次发送停止命令
                    stopCommandCount.value = 2

                    delay(50) // 再次延时50ms
                    viewModel.stop() // 第三次发送停止命令
                    stopCommandCount.value = 0 // 重置计数器
                }
            }
        } else if ((state.x != lastSentState.value.x || state.y != lastSentState.value.y) &&
            currentTime - lastSentTime.value >= 50
        ) {
            // 非停止命令的正常处理
            stopCommandCount.value = 0 // 重置停止命令计数器
            val x = (state.x * 100).toInt()
            val y = (state.y * 100).toInt()
            viewModel.sendXYR(x, y, 0)

            // 更新上次发送的状态和时间
            lastSentState.value = state
            lastSentTime.value = currentTime
        }
    }
}