package icu.fur93.esp32_car.page

import DebugWindowManager
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Icon
import androidx.compose.material3.ListItem
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.vectorResource
import icu.fur93.esp32_car.BuildConfig
import icu.fur93.esp32_car.R
import icu.fur93.esp32_car.Route
import icu.fur93.esp32_car.repository.BluetoothRepositoryImpl
import icu.fur93.esp32_car.repository.ConnectionState
import icu.fur93.esp32_car.ui.component.PageTitle
import icu.fur93.esp32_car.ui.component.TipText
import icu.fur93.esp32_car.ui.dialog.BleDeviceScanDialog
import icu.fur93.esp32_car.ui.theme.LayoutContentPadding
import icu.fur93.esp32_car.ui.theme.LayoutTopPadding
import icu.fur93.esp32_car.viewmodel.CarViewModel

@Composable
fun SettingsPage(viewModel: CarViewModel) {
    Column {
        Column(
            Modifier.padding(
                top = LayoutTopPadding,
                start = LayoutContentPadding,
                end = LayoutContentPadding
            )
        ) {
            PageTitle(Route.Settings.title)
        }
        SettingsList(viewModel)
        TipText("GitHub: colour93/esp32-car-android")
        TipText(
            "Build: ${
                android.text.format.DateFormat.format(
                    "yyyy/MM/dd HH:mm:ss",
                    BuildConfig.BUILD_TIME.toLong()
                )
            }"
        )
    }
}

@Composable
fun SettingsList(viewModel: CarViewModel) {
    val context = LocalContext.current

    var debugMode by remember { mutableStateOf(DebugWindowManager.isShowing()) }

    val onDebugModeChange = { enable: Boolean ->
        if (enable) {
            DebugWindowManager.show(context, viewModel)
        } else {
            DebugWindowManager.hide(context)
        }
        debugMode = enable
    }

    // 使用 LazyColumn 显示设置列表
    LazyColumn {
        item {
            SettingConnectDeviceItem(viewModel)
        }
        item {
            SettingDeviceNameItem(viewModel)
        }
        item {
            ListItem(
                headlineContent = { Text("调试模式") },
                trailingContent = {
                    Switch(
                        checked = debugMode,
                        onCheckedChange = onDebugModeChange
                    )
                }
            )
        }
        item {
            ListItem(
                headlineContent = { Text("PID 参数") },
                trailingContent = {
                    Icon(
                        ImageVector.vectorResource(R.drawable.arrow_right_24),
                        contentDescription = "PID 参数"
                    )
                }
            )
        }
    }
}


@Composable
fun SettingConnectDeviceItem(viewModel: CarViewModel) {
    var showScanDialog by remember { mutableStateOf(false) }
    var showDisconnectDialog by remember { mutableStateOf(false) }

    val connectionState by viewModel.connectionState.collectAsState()
    val connectionInfoState by viewModel.connectionInfoState.collectAsState()


    ListItem(
        headlineContent = { Text("连接设备") },
        supportingContent = {
            if (connectionState == ConnectionState.CONNECTED) Text("已连接") else Text(
                "未连接"
            )
        },
        trailingContent = {
            Icon(
                ImageVector.vectorResource(R.drawable.arrow_right_24),
                contentDescription = "连接设备"
            )
        },
        modifier = Modifier.clickable {
            if (connectionState == ConnectionState.CONNECTED) {
                showDisconnectDialog = true
            } else if (connectionState == ConnectionState.DISCONNECTED) {
                showScanDialog = true
            }
        }
    )

    // 扫描设备对话框
    BleDeviceScanDialog(
        isVisible = showScanDialog,
        onDismiss = { showScanDialog = false },
        viewModel = viewModel
    )

    // 断开连接确认对话框
    if (showDisconnectDialog) {
        AlertDialog(
            onDismissRequest = { showDisconnectDialog = false },
            title = { Text("断开连接") },
            text = { Text("确定要断开与 ${connectionInfoState.name} 的连接吗？") },
            confirmButton = {
                TextButton(
                    onClick = {
                        (viewModel.repository as BluetoothRepositoryImpl).disconnect()
                        showDisconnectDialog = false
                    }
                ) {
                    Text("确定")
                }
            },
            dismissButton = {
                TextButton(
                    onClick = { showDisconnectDialog = false }
                ) {
                    Text("取消")
                }
            }
        )
    }
}

@Composable
fun SettingDeviceNameItem(viewModel: CarViewModel) {
    val connectionInfoState by viewModel.connectionInfoState.collectAsState()

    ListItem(
        headlineContent = { Text("设备名称") },
        supportingContent = { Text("当前名称: ${connectionInfoState.name}") },
        trailingContent = {
            Icon(
                ImageVector.vectorResource(R.drawable.arrow_right_24),
                contentDescription = "设备名称"
            )
        }
    )
}
