package icu.fur93.esp32_car.repository

import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothDevice.TRANSPORT_LE
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothGattDescriptor
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothProfile
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.Context
import android.util.Log
import android.widget.Toast
import icu.fur93.esp32_car.const.CarCommands
import icu.fur93.esp32_car.entity.BleDevice
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import java.util.UUID
import icu.fur93.esp32_car.data.PreferencesDataStore
import icu.fur93.esp32_car.entity.LogDirection
import icu.fur93.esp32_car.entity.LogEntry
import icu.fur93.esp32_car.entity.car.CarState
import icu.fur93.esp32_car.entity.car.InfraredState
import icu.fur93.esp32_car.entity.car.MotorState

// 蓝牙通信接口
interface BluetoothRepository {
    fun sendCommand(command: ByteArray)
    fun observeCarState(): Flow<CarState>
    fun observeLogs(): Flow<List<LogEntry>>
    fun clearLogs()
    val connectionState: StateFlow<ConnectionState>
    val deviceName: StateFlow<String?>
    val deviceAddress: StateFlow<String?>
}

// 添加连接状态枚举
enum class ConnectionState {
    DISCONNECTED,
    CONNECTING,
    CONNECTED,
    DISCONNECTING
}

val serviceUUID = "6E400001-B5A3-F393-E0A9-E50E24DCCA9E"
val rxCharUUID = "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
val txCharUUID = "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"

val deviceInfoServiceUUID = "0000180a-0000-1000-8000-00805f9b34fb"
val deviceNameCharUUID = "00002a00-0000-1000-8000-00805f9b34fb"

// 蓝牙通信实现
class BluetoothRepositoryImpl(
    private val context: Context,
    private val scope: CoroutineScope,
    private val preferencesDataStore: PreferencesDataStore
) : BluetoothRepository {
    private var bluetoothGatt: BluetoothGatt? = null
    private var rxCharacteristic: BluetoothGattCharacteristic? = null

    private val _carState = MutableStateFlow(CarState())
    private val _logs = MutableStateFlow<List<LogEntry>>(emptyList())

    private val bluetoothAdapter: BluetoothAdapter? =
        (context.getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter

    private val _scanResults = MutableStateFlow<List<BleDevice>>(emptyList())
    val scanResults: StateFlow<List<BleDevice>> = _scanResults.asStateFlow()

    private val _isScanning = MutableStateFlow(false)
    val isScanning: StateFlow<Boolean> = _isScanning.asStateFlow()

    // 连接状态流
    private val _connectionState = MutableStateFlow(ConnectionState.DISCONNECTED)
    override val connectionState: StateFlow<ConnectionState> = _connectionState.asStateFlow()

    private val _deviceName = MutableStateFlow<String?>(null)
    override val deviceName: StateFlow<String?> = _deviceName.asStateFlow()

    private val _deviceAddress = MutableStateFlow<String?>(null)
    override val deviceAddress: StateFlow<String?> = _deviceAddress.asStateFlow()

    private val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            val bleDevice = BleDevice(
                device = result.device,
                rssi = result.rssi,
                scanRecord = result.scanRecord?.bytes
            )

            val currentList = _scanResults.value.toMutableList()
            val index = currentList.indexOfFirst { it.device.address == bleDevice.device.address }

            if (index >= 0) {
                currentList[index] = bleDevice
            } else {
                currentList.add(bleDevice)
            }

            // 按信号强度排序
            _scanResults.value = currentList.sortedByDescending { it.rssi }
        }
    }

    @SuppressLint("MissingPermission")
    fun startScan() {
        if (_isScanning.value) return

        _scanResults.value = emptyList()
        _isScanning.value = true

        val scanner = bluetoothAdapter?.bluetoothLeScanner
        val settings = ScanSettings.Builder()
            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
            .build()

        scanner?.startScan(null, settings, scanCallback)

        // 10秒后自动停止扫描
        scope.launch {
            delay(10000)
            stopScan()
        }
    }

    @SuppressLint("MissingPermission")
    fun stopScan() {
        if (!_isScanning.value) return

        _isScanning.value = false
        bluetoothAdapter?.bluetoothLeScanner?.stopScan(scanCallback)
    }

    @SuppressLint("MissingPermission")
    fun connectToDevice(
        device: BluetoothDevice,
        callback: ((Boolean, BluetoothDevice?) -> Unit)? = null
    ) {
        _connectionState.value = ConnectionState.CONNECTING
        device.connectGatt(
            context,
            false,
            object : BluetoothGattCallback() {
                override fun onConnectionStateChange(
                    gatt: BluetoothGatt,
                    status: Int,
                    newState: Int
                ) {
                    Log.d("onConnectionStateChange", "status: $status, newState: $newState")

                    if (status != BluetoothGatt.GATT_SUCCESS) {
                        scope.launch(Dispatchers.Main) {
                            Toast.makeText(context, "连接失败，错误码: $status", Toast.LENGTH_SHORT)
                                .show()
                        }
                    }
                    when (newState) {
                        BluetoothProfile.STATE_CONNECTED -> {
                            bluetoothGatt = gatt
                            _connectionState.value = ConnectionState.CONNECTED
                            scope.launch(Dispatchers.Main) {
                                gatt.discoverServices()
                                _connectionState.collect { state ->
                                    if (state == ConnectionState.CONNECTED) {
                                        preferencesDataStore.saveLastConnectedDevice(device.address)
                                    }
                                }
                            }
                            if (callback != null) {
                                callback(true, gatt.device)
                            }
                            _deviceAddress.value = gatt.device.address
                            _deviceName.value = gatt.device.name ?: "未知设备"
                        }

                        BluetoothProfile.STATE_CONNECTING -> {
                            _connectionState.value = ConnectionState.CONNECTING
                            if (callback != null) {
                                callback(false, null)
                            }
                        }

                        BluetoothProfile.STATE_DISCONNECTING -> {
                            _connectionState.value = ConnectionState.DISCONNECTING
                            if (callback != null) {
                                callback(false, null)
                            }
                        }

                        BluetoothProfile.STATE_DISCONNECTED -> {
                            bluetoothGatt = null
                            rxCharacteristic = null
                            _connectionState.value = ConnectionState.DISCONNECTED
                            if (callback != null) {
                                callback(false, null)
                            }
                            _deviceName.value = null
                            _deviceAddress.value = null
                        }
                    }
                }

                override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
                    super.onServicesDiscovered(gatt, status)
                    if (status == BluetoothGatt.GATT_SUCCESS) {
                        // 读取设备名称
                        val genericService =
                            gatt?.getService(UUID.fromString("00001800-0000-1000-8000-00805f9b34fb"))
                        val deviceNameChar =
                            genericService?.getCharacteristic(UUID.fromString("00002a00-0000-1000-8000-00805f9b34fb"))
                        if (deviceNameChar != null) {
                            gatt.readCharacteristic(deviceNameChar)
                        }

                        gatt?.getService(UUID.fromString(serviceUUID))?.let { service ->
                            bluetoothGatt = gatt
                            rxCharacteristic =
                                service.getCharacteristic(UUID.fromString(rxCharUUID))

                            // 注册通知监听器
                            val txCharacteristic =
                                service.getCharacteristic(UUID.fromString(txCharUUID))
                            gatt.setCharacteristicNotification(txCharacteristic, true)
                            txCharacteristic.descriptors.forEach { descriptor ->
                                descriptor.value = BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
                                gatt.writeDescriptor(descriptor)
                            }
                        }
                    }
                }

                override fun onCharacteristicRead(
                    gatt: BluetoothGatt,
                    characteristic: BluetoothGattCharacteristic,
                    value: ByteArray,
                    status: Int
                ) {
                    super.onCharacteristicRead(gatt, characteristic, value, status)
                    if (characteristic.uuid == UUID.fromString(deviceNameCharUUID)) {
                        val deviceName = String(value)
                        _deviceName.value = deviceName
                        Log.d("DeviceName", "Device name: $deviceName")
                    }
                }

                override fun onCharacteristicChanged(
                    gatt: BluetoothGatt,
                    characteristic: BluetoothGattCharacteristic,
                    value: ByteArray
                ) {
                    super.onCharacteristicChanged(gatt, characteristic, value)
                    if (characteristic.uuid == UUID.fromString(txCharUUID)) {
                        onReceivePacket(value)
                    }
                    if (characteristic.uuid == UUID.fromString(deviceNameCharUUID)) {
                        val deviceName = String(value)
                        _deviceName.value = deviceName
                        _deviceAddress.value = gatt.device.address
                    }
                }
            },
            TRANSPORT_LE
        )
    }

    @SuppressLint("MissingPermission")
    fun connectToDeviceByAddress(
        address: String,
        callback: ((Boolean, BluetoothDevice) -> Unit)? = null
    ) {
        val bluetoothManager =
            context.getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
        val device = bluetoothManager.adapter.getRemoteDevice(address)
        if (callback != null) {
            connectToDevice(device) { isConnected, deviceDetail ->
                callback(isConnected, deviceDetail ?: device)
            }
        } else {
            connectToDevice(device)
        }

    }

    @SuppressLint("MissingPermission")
    private fun setupDeviceNameNotification(gatt: BluetoothGatt) {
        val service = gatt.getService(UUID.fromString(deviceInfoServiceUUID))
        val characteristic = service?.getCharacteristic(UUID.fromString(deviceNameCharUUID))

        characteristic?.let { char ->
            // 启用通知
            gatt.setCharacteristicNotification(char, true)

            // 获取 descriptor 并写入通知使能值
            val descriptor =
                char.getDescriptor(UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"))
            descriptor?.let {
                it.value = BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
                gatt.writeDescriptor(it)
            }

            // 同时也读取一次当前值
            gatt.readCharacteristic(char)
        }
    }

    @SuppressLint("MissingPermission")
    override fun sendCommand(command: ByteArray) {
        rxCharacteristic?.let { characteristic ->
            characteristic.value = command
            bluetoothGatt?.writeCharacteristic(characteristic)
            addLog(LogDirection.SEND, command)
        }
    }

    override fun observeCarState(): Flow<CarState> = _carState.asStateFlow()
    override fun observeLogs(): Flow<List<LogEntry>> = _logs.asStateFlow()
    override fun clearLogs() {
        _logs.value = emptyList()
    }

    private fun addLog(direction: LogDirection, data: ByteArray) {
        _logs.value += LogEntry(
            direction = direction,
            data = data.joinToString(" ") { "0x%02X".format(it) }
        )
    }

    fun onReceivePacket(packet: ByteArray) {
        addLog(LogDirection.RECEIVE, packet)
        parsePacket(packet)
    }

    private fun parsePacket(packet: ByteArray) {
        if (!isValidPacket(packet)) return

        /**
        包体 `01 0E E0 电机A_IN_1_2 电机A_PWM 电机B_IN_1_2 电机B_PWM 电机C_IN_1_2 电机C_PWM 电机D_IN_1_2 电机D_PWM 红外引脚数量 红外数据 FE`

        | 数据 | 类型 | 表示 |
        | -------- | ---- | ---- |
        | IN_1_2  | 无符号整数 | 按位表示 IN1 IN2 的值， 0000 00 IN2 IN1 |
        | PWM     | 无符号整数 | - |
        | 红外引脚数量 | 无符号整数 | - |
        | 红外数据 | 无符号整数 | 按位表示红外循迹模块数据， 0001 1111 |

        示例 `01 0E E0 01 FF 02 FF 02 FF 01 FF 05 1F FE`
         **/

        when (packet[2].toUByte().toUInt()) {
            CarCommands.CMD_STATUS -> {
                updateStatus(packet)
            }
        }
    }

    private fun isValidPacket(packet: ByteArray): Boolean =
        packet[0].toUByte() == CarCommands.PACKET_R_HEAD.toUByte() &&
                packet.size == packet[1].toUByte().toInt() &&
                packet[packet[1].toUByte()
                    .toInt() - 1].toUByte() == CarCommands.PACKET_R_TAIL.toUByte()

    private fun updateStatus(packet: ByteArray) {
        val data = packet.sliceArray(3 until packet[1].toUByte().toInt() - 1)
        _carState.value = _carState.value.copy(
            motorAState = createMotorState(data, 0),
            motorBState = createMotorState(data, 2),
            motorCState = createMotorState(data, 4),
            motorDState = createMotorState(data, 6),
            infraredState = createInfraredState(data[8].toUByte().toInt(), data[9].toUByte())
        )
    }

    private fun createMotorState(data: ByteArray, offset: Int) = MotorState(
        pwm = data[offset + 1].toUByte().toUInt(),
        in1 = data[offset].toUByte().toUInt() and 1u,
        in2 = (data[offset].toUByte().toUInt() shr 1) and 1u
    )

    private fun createInfraredState(irPinCount: Int, irData: UByte) = InfraredState(
        enable = true,
        statusList = irData.toString(2).padStart(irPinCount, '0').reversed()
            .map { if (it == '1') 1u else 0u }
    )

    // 断开连接方法
    @SuppressLint("MissingPermission")
    fun disconnect() {
        bluetoothGatt?.let { gatt ->
            _connectionState.value = ConnectionState.DISCONNECTING
            gatt.disconnect()
            gatt.close() // 重要：需要调用 close() 释放资源
            bluetoothGatt = null
            rxCharacteristic = null
            _deviceName.value = null
            _deviceAddress.value = null
            _connectionState.value = ConnectionState.DISCONNECTED
        }
    }
}