package icu.fur93.esp32_car.ui.component

import androidx.compose.foundation.Canvas
import androidx.compose.foundation.gestures.detectDragGestures
import androidx.compose.foundation.layout.*
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.drawscope.DrawScope
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import icu.fur93.esp32_car.entity.control.JoystickState
import kotlin.math.atan2
import kotlin.math.hypot

// 摇杆数据类
@Composable
fun Joystick(
    modifier: Modifier = Modifier,
    size: Dp = 200.dp,
    knobScale: Float = 0.4f,
    onJoystickMove: (JoystickState) -> Unit
) {
    // 状态管理
    var centerPoint by remember { mutableStateOf(Offset.Zero) }
    var stickPosition by remember { mutableStateOf(Offset.Zero) }
    var isDragging by remember { mutableStateOf(false) }

    val surfaceColor = MaterialTheme.colorScheme.surfaceDim
    val primaryColor = MaterialTheme.colorScheme.primary

    // 创建摇杆
    Canvas(
        modifier = modifier
            .size(size)
            .pointerInput(Unit) {
                // 将 size 转换为像素
                val sizePx = size.toPx()
                val radius = sizePx / 2
                val innerRadius = radius * knobScale

                detectDragGestures(
                    onDragStart = { offset ->
                        isDragging = true
                        centerPoint = Offset(sizePx / 2, sizePx / 2)
                        updateStickPosition(offset, centerPoint, radius, innerRadius) { newPos ->
                            stickPosition = newPos
                            val state = calculateJoystickState(centerPoint, newPos, radius)
                            onJoystickMove(state)
                        }
                    },
                    onDrag = { change, _ ->
                        updateStickPosition(change.position, centerPoint, radius, innerRadius) { newPos ->
                            stickPosition = newPos
                            val state = calculateJoystickState(centerPoint, newPos, radius)
                            onJoystickMove(state)
                        }
                    },
                    onDragEnd = {
                        isDragging = false
                        // 回到中心位置
                        stickPosition = centerPoint
                        onJoystickMove(JoystickState())
                    }
                )
            }
    ) {
        // 获取实际像素尺寸
        val sizePx = size.toPx()
        val radius = sizePx / 2
        val innerRadius = radius * knobScale

        // 初始化中心点
        if (centerPoint == Offset.Zero) {
            centerPoint = Offset(sizePx / 2, sizePx / 2)
            stickPosition = centerPoint
        }

        // 绘制底部圆形
        drawJoystickBase(centerPoint, radius, surfaceColor)

        // 绘制摇杆
        drawJoystickKnob(stickPosition, innerRadius, primaryColor)
    }
}

// 更新摇杆位置
private fun updateStickPosition(
    pointerPosition: Offset,
    centerPoint: Offset,
    maxRadius: Float,
    knobRadius: Float,
    onUpdate: (Offset) -> Unit
) {
    val distance = hypot(
        pointerPosition.x - centerPoint.x,
        pointerPosition.y - centerPoint.y
    )

    val newPosition = if (distance > maxRadius) {
        // 限制在最大范围内
        val angle = atan2(
            pointerPosition.y - centerPoint.y,
            pointerPosition.x - centerPoint.x
        )
        Offset(
            centerPoint.x + (maxRadius * kotlin.math.cos(angle)),
            centerPoint.y + (maxRadius * kotlin.math.sin(angle))
        )
    } else {
        pointerPosition
    }

    onUpdate(newPosition)
}

// 计算摇杆状态
private fun calculateJoystickState(
    centerPoint: Offset,
    stickPosition: Offset,
    maxRadius: Float
): JoystickState {
    val deltaX = centerPoint.x - stickPosition.x
    val deltaY = centerPoint.y - stickPosition.y
    val distance = hypot(deltaX, deltaY)
    val angle = atan2(deltaY, deltaX)

    return JoystickState(
        x = (deltaX / maxRadius).coerceIn(-1f, 1f),
        y = (deltaY / maxRadius).coerceIn(-1f, 1f),
        angle = angle.toFloat(),
        distance = (distance / maxRadius).coerceIn(0f, 1f)
    )
}

// 绘制底部圆形
private fun DrawScope.drawJoystickBase(center: Offset, radius: Float, color: Color) {
    drawCircle(
        color = color,
        radius = radius,
        center = center
    )
}

// 绘制摇杆球
private fun DrawScope.drawJoystickKnob(position: Offset, radius: Float, color: Color) {
    drawCircle(
        color = color,
        radius = radius,
        center = position
    )
}

// 格式化浮点数
fun Float.format(digits: Int) = "%.${digits}f".format(this)