package icu.fur93.esp32_car.viewmodel

import android.annotation.SuppressLint
import android.bluetooth.BluetoothDevice
import android.util.Log
import androidx.compose.ui.platform.ComposeView
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import icu.fur93.esp32_car.const.CarCommands
import icu.fur93.esp32_car.entity.BleDevice
import icu.fur93.esp32_car.entity.LogEntry
import icu.fur93.esp32_car.entity.car.CarState
import icu.fur93.esp32_car.entity.car.ConnectionInfoState
import icu.fur93.esp32_car.repository.BluetoothRepository
import icu.fur93.esp32_car.repository.BluetoothRepositoryImpl
import icu.fur93.esp32_car.repository.ConnectionState
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import android.view.WindowManager
import kotlinx.coroutines.flow.combine


// 用例
class CarControlUseCase(
    private val repository: BluetoothRepository
) {
    fun moveForward(speed: Int = 255) = repository.sendCommand(
        buildCommand(CarCommands.CMD_MOTOR_MOVE_CONTROL, 0x01, speed)
    )

    fun moveBackward(speed: Int = 255) = repository.sendCommand(
        buildCommand(CarCommands.CMD_MOTOR_MOVE_CONTROL, 0x02, speed)
    )

    fun turnLeft(speed: Int = 255) = repository.sendCommand(
        buildCommand(CarCommands.CMD_MOTOR_STEER_CONTROL, 0x00, speed)
    )

    fun turnRight(speed: Int = 255) = repository.sendCommand(
        buildCommand(CarCommands.CMD_MOTOR_STEER_CONTROL, 0x01, speed)
    )

    fun stop() = repository.sendCommand(
        buildCommand(CarCommands.CMD_MOTOR_MOVE_CONTROL, 0x00, 0x00)
    )

    fun sendXYR(x: Int, y: Int, r: Int) = repository.sendCommand(
        byteArrayOf(
            CarCommands.PACKET_T_HEAD.toByte(),
            0x07,
            CarCommands.CMD_MOTOR_XYR_CONTROL.toByte(),
            x.toByte(),
            y.toByte(),
            r.toByte(),
            CarCommands.PACKET_T_TAIL.toByte()
        )
    )

    private fun buildCommand(cmd: UInt, param1: Int, param2: Int) = byteArrayOf(
        CarCommands.PACKET_T_HEAD.toByte(),
        0x06,
        cmd.toByte(),
        param1.toByte(),
        param2.toByte(),
        CarCommands.PACKET_T_TAIL.toByte()
    )
}

class CarViewModel(
    private val carControlUseCase: CarControlUseCase,
    val repository: BluetoothRepository
) : ViewModel() {
    val scanResults: StateFlow<List<BleDevice>> =
        (repository as BluetoothRepositoryImpl).scanResults

    val isScanning: StateFlow<Boolean> =
        (repository as BluetoothRepositoryImpl).isScanning

    fun startScan() = (repository as BluetoothRepositoryImpl).startScan()
    fun stopScan() = (repository as BluetoothRepositoryImpl).stopScan()

    @SuppressLint("MissingPermission")
    fun connectToDevice(device: BluetoothDevice) {
        (repository as BluetoothRepositoryImpl).connectToDevice(device)
    }

    @SuppressLint("MissingPermission")
    fun connectToDeviceByAddress(address: String) {
        (repository as BluetoothRepositoryImpl).connectToDeviceByAddress(address)
    }

    val carState: StateFlow<CarState> = repository.observeCarState()
        .stateIn(viewModelScope, SharingStarted.WhileSubscribed(), CarState())

    val logs: StateFlow<List<LogEntry>> = repository.observeLogs()
        .stateIn(viewModelScope, SharingStarted.WhileSubscribed(), emptyList())

    // 设备信息状态
    private val _connectionInfoState = MutableStateFlow(ConnectionInfoState())
    val connectionInfoState: StateFlow<ConnectionInfoState> = _connectionInfoState.asStateFlow()

    val connectionState: StateFlow<ConnectionState> =
        (repository as BluetoothRepositoryImpl).connectionState
            .stateIn(
                scope = viewModelScope,
                started = SharingStarted.WhileSubscribed(5000),
                initialValue = ConnectionState.DISCONNECTED
            )

    // 添加断开连接方法
    fun disconnect() {
        (repository as BluetoothRepositoryImpl).disconnect()
    }

    init {
        viewModelScope.launch {
            // 监听连接状态变化
            connectionState.collect { state ->
                when (state) {
                    ConnectionState.DISCONNECTED -> {
                        // 断开连接时清空设备信息
                        _connectionInfoState.value = ConnectionInfoState()
                    }
                    else -> {
                        // 其他状态处理...
                    }
                }
            }
        }

        viewModelScope.launch {
            repository.deviceName.combine(repository.deviceAddress) { name, address ->
                Pair(name, address)
            }.collect { (name, address) ->
                if (name != null && address != null) {
                    updateDeviceInfo(name, address)
                }
            }
        }
    }

    // 更新设备信息的方法
    private fun updateDeviceInfo(deviceName: String?, deviceAddress: String) {
        _connectionInfoState.value = ConnectionInfoState(
            name = deviceName ?: "未知设备 (${deviceAddress})",
            address = deviceAddress,
            version = "001"
        )
    }

    fun moveForward(speed: Int = 255) = carControlUseCase.moveForward(speed)
    fun moveBackward(speed: Int = 255) = carControlUseCase.moveBackward(speed)
    fun turnLeft(speed: Int = 255) = carControlUseCase.turnLeft(speed)
    fun turnRight(speed: Int = 255) = carControlUseCase.turnRight(speed)
    fun stop() = carControlUseCase.stop()
    fun sendXYR(x: Int, y: Int, r: Int) = carControlUseCase.sendXYR(x, y, r)
    fun clearLogs() = repository.clearLogs()
}