package icu.fur93.esp32_car.window

import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.detectDragGestures
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.FilterChip
import androidx.compose.material3.FilterChipDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.res.vectorResource
import androidx.compose.ui.unit.dp
import icu.fur93.esp32_car.R
import icu.fur93.esp32_car.entity.LogDirection
import icu.fur93.esp32_car.entity.LogEntry
import java.text.SimpleDateFormat
import java.util.Locale

@Composable
fun DebugWindow(
    onDrag: (Float, Float) -> Unit,
    onResize: (Float, Float) -> Unit,
    logs: List<LogEntry>
) {
    var logFilter by remember { mutableStateOf(LogFilter.ALL) }

    Box(
        modifier = Modifier
            .width(280.dp)
            .height(200.dp)
            .background(
                color = Color.Black.copy(alpha = 0.8f),
                shape = RoundedCornerShape(8.dp)
            )
    ) {
        Column {
            // 顶部拖动条
            Box(
                modifier = Modifier
                    .fillMaxWidth()
                    .height(32.dp)
                    .background(
                        color = Color.DarkGray,
                        shape = RoundedCornerShape(topStart = 8.dp, topEnd = 8.dp)
                    )
                    .pointerInput(Unit) {
                        detectDragGestures { change, dragAmount ->
                            change.consume()
                            onDrag(dragAmount.x, dragAmount.y)
                        }
                    }
            ) {
                Text(
                    "Debug Window",
                    color = Color.White,
                    style = MaterialTheme.typography.titleSmall,
                    modifier = Modifier
                        .padding(horizontal = 12.dp, vertical = 6.dp)
                        .align(Alignment.CenterStart)
                )
            }

            // 过滤选项
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .background(Color.DarkGray.copy(alpha = 0.3f))
                    .padding(horizontal = 8.dp, vertical = 4.dp),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                FilterChip(
                    selected = logFilter == LogFilter.ALL,
                    onClick = { logFilter = LogFilter.ALL },
                    label = { Text("全部", color = Color.White) },
                    colors = FilterChipDefaults.filterChipColors(
                        selectedContainerColor = MaterialTheme.colorScheme.primary
                    )
                )
                FilterChip(
                    selected = logFilter == LogFilter.SEND,
                    onClick = { logFilter = LogFilter.SEND },
                    label = { Text("发送", color = Color.White) },
                    colors = FilterChipDefaults.filterChipColors(
                        selectedContainerColor = MaterialTheme.colorScheme.primary
                    )
                )
                FilterChip(
                    selected = logFilter == LogFilter.RECEIVE,
                    onClick = { logFilter = LogFilter.RECEIVE },
                    label = { Text("接收", color = Color.White) },
                    colors = FilterChipDefaults.filterChipColors(
                        selectedContainerColor = MaterialTheme.colorScheme.primary
                    )
                )
            }

            // 日志列表
            LazyColumn(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(horizontal = 8.dp),
                reverseLayout = true // 最新的日志显示在顶部
            ) {
                items(
                    logs.filter {
                        when (logFilter) {
                            LogFilter.ALL -> true
                            LogFilter.SEND -> it.direction == LogDirection.SEND
                            LogFilter.RECEIVE -> it.direction == LogDirection.RECEIVE
                        }
                    }.reversed()
                ) { log ->
                    LogItem(log)
                }
            }
        }

        // 右下角调整大小的手柄
        Box(
            modifier = Modifier
                .size(24.dp)
                .align(Alignment.BottomEnd)
                .padding(4.dp)
                .pointerInput(Unit) {
                    detectDragGestures { change, dragAmount ->
                        change.consume()
                        onResize(dragAmount.x, dragAmount.y)
                    }
                }
        ) {
            Icon(
                imageVector = ImageVector.vectorResource(R.drawable.resize_24),
                contentDescription = "Resize",
                tint = Color.White.copy(alpha = 0.6f),
                modifier = Modifier.size(20.dp)
            )
        }
    }
}

@Composable
private fun LogItem(log: LogEntry) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = 2.dp),
        horizontalArrangement = Arrangement.SpaceBetween
    ) {
        // 方向指示
        Text(
            text = if (log.direction == LogDirection.SEND) "→" else "←",
            color = if (log.direction == LogDirection.SEND) 
                Color(0xFF4CAF50) else Color(0xFF2196F3),
            style = MaterialTheme.typography.bodySmall,
            modifier = Modifier.width(16.dp)
        )
        // 日志内容
        Text(
            text = log.data,
            color = Color.White,
            style = MaterialTheme.typography.bodySmall,
            modifier = Modifier.weight(1f)
        )
        // 时间戳
        Text(
            text = SimpleDateFormat("HH:mm:ss", Locale.getDefault())
                .format(log.timestamp),
            color = Color.Gray,
            style = MaterialTheme.typography.bodySmall
        )
    }
}

private enum class LogFilter {
    ALL, SEND, RECEIVE
}