import android.content.Context
import android.graphics.PixelFormat
import android.view.Gravity
import android.view.WindowManager
import androidx.activity.ComponentActivity
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.platform.ComposeView
import androidx.lifecycle.setViewTreeLifecycleOwner
import androidx.savedstate.setViewTreeSavedStateRegistryOwner
import icu.fur93.esp32_car.window.DebugWindow
import icu.fur93.esp32_car.viewmodel.CarViewModel

object DebugWindowManager {
    private var debugWindow: ComposeView? = null
    private var layoutParams: WindowManager.LayoutParams? = null
    
    // 添加最小尺寸限制
    private const val MIN_WIDTH_DP = 200
    private const val MIN_HEIGHT_DP = 150

    fun show(context: Context, viewModel: CarViewModel) {
        if (debugWindow != null) return
        
        val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        val activity = context as? ComponentActivity ?: return
        
        val density = context.resources.displayMetrics.density
        val width = (280 * density).toInt()
        val height = (200 * density).toInt()
        
        val params = WindowManager.LayoutParams(
            width,
            height,
            WindowManager.LayoutParams.TYPE_APPLICATION,
            WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE,
            PixelFormat.TRANSLUCENT
        ).apply {
            gravity = Gravity.TOP or Gravity.START
            x = 100
            y = 100
        }
        
        val window = ComposeView(context).apply {
            setViewTreeLifecycleOwner(activity)
            setViewTreeSavedStateRegistryOwner(activity)
            
            setContent {
                val logs by viewModel.logs.collectAsState()
                
                DebugWindow(
                    onDrag = { offsetX, offsetY ->
                        params.x += offsetX.toInt()
                        params.y += offsetY.toInt()
                        windowManager.updateViewLayout(this, params)
                    },
                    onResize = { offsetX, offsetY ->
                        val density = context.resources.displayMetrics.density
                        val minWidth = (MIN_WIDTH_DP * density).toInt()
                        val minHeight = (MIN_HEIGHT_DP * density).toInt()
                        params.width = (params.width + offsetX.toInt()).coerceAtLeast(minWidth)
                        params.height = (params.height + offsetY.toInt()).coerceAtLeast(minHeight)
                        windowManager.updateViewLayout(this, params)
                    },
                    logs = logs
                )
            }
        }
        
        windowManager.addView(window, params)
        debugWindow = window
        layoutParams = params
    }
    
    fun hide(context: Context) {
        val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        debugWindow?.let {
            windowManager.removeView(it)
            debugWindow = null
            layoutParams = null
        }
    }
    
    fun isShowing(): Boolean {
        return debugWindow != null
    }
    
    fun updatePosition(context: Context, x: Int, y: Int) {
        val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        layoutParams?.let { params ->
            params.x = x
            params.y = y
            debugWindow?.let {
                windowManager.updateViewLayout(it, params)
            }
        }
    }
} 